DATADIR=~/bench-indexes/
OUT_DIR=./benchdata
OUTFILE=${OUT_DIR}/index_outputs.log
num_iterations=7
num_transactions=3

function log() {
  line="$(date --iso-8601=seconds) $1"
  echo $line;
  echo $line >> OUTFILE 
}


for pgversion in 14_master 14_0001 14_0002 14_0007; do
  BINDIR=./${pgversion}_pg_install/bin
  LOGFILE=~/bench-indexes/postgres-${pgversion}.log

  log "Starting with ${pgversion}, logfile=$LOGFILE, bindir=$BINDIR, stdout=$STDOUT";

  for dataset in sample_data_100k sample_data_1m; do
    ${BINDIR}/pg_ctl -D ${DATADIR} -l ${LOGFILE} start >> ${OUTFILE}
    psql -h localhost --port 5432 testing -c "ALTER TABLE ${dataset} RENAME TO source; SELECT pg_prewarm('source'::regclass)" >> ${OUTFILE};

    for bench_iteration in $(seq 1 ${num_iterations}); do
      STDOUT=${OUT_DIR}/pgbench_stats_${pgversion}_${dataset}_${bench_iteration}.out

      log "Starting iteration ${bench_iteration} of ${num_iterations} for ${dataset} in ${pgversion}"
      ${BINDIR}/pgbench \
          --host localhost \
          --port 5432 \
          --file bench_indexes.sql \
          --client 1 \
          --jobs 1 \
          --log \
          --log-prefix ${OUT_DIR}/pgbench_logs_${pgversion}_${dataset}_${bench_iteration}.log \
          --no-vacuum \
          --report-per-command \
          --transactions ${num_transactions} \
          testing > "${STDOUT}"
    done;

    psql -h localhost --port 5432 testing -c "ALTER TABLE source RENAME TO ${dataset}" >> ${OUTFILE};

    log "Done with dataset ${dataset} on ${pgversion}";

    ${BINDIR}/pg_ctl -D ${DATADIR} -l ${LOGFILE} stop >> ${OUTFILE};
  done;

  log "Done with ${pgversion}"
done;
