BEGIN;
-- single fixed-size column (offset fully cacheable)
CREATE UNIQUE INDEX uidx ON target USING btree (transaction);
INSERT INTO target TABLE source /* uidx */;
REINDEX INDEX uidx;
DROP INDEX uidx;
TRUNCATE target;

-- single fixed-size column (offset fully cacheable)
CREATE INDEX nuidx ON target USING btree (country);
INSERT INTO target TABLE source /* nuidx */;
REINDEX INDEX nuidx;
DROP INDEX nuidx;
TRUNCATE target;

-- var-len prefix column (no offset cache after 1st column)
CREATE INDEX ccl ON target USING btree (country, city, locality);
INSERT INTO target TABLE source /* ccl */;
REINDEX INDEX ccl;
DROP INDEX ccl;
TRUNCATE target;

-- unique, const-len prefix (offset cache usable)
CREATE INDEX ul ON target USING btree (transaction, locality);
INSERT INTO target TABLE source /* ul */;
REINDEX INDEX ul;
DROP INDEX ul;
TRUNCATE target;

-- var-len prefix column (no offset cache after 1st column)
CREATE INDEX lu ON target USING btree (locality, transaction);
INSERT INTO target TABLE source /* lu */;
REINDEX INDEX lu;
DROP INDEX lu;
TRUNCATE target;

-- var-len prefix column with (expensive) collation, no cache after 1st column
CREATE INDEX ccl_collated
    ON target
        USING btree (country COLLATE "en_US",
                     city COLLATE "en_US",
                     locality COLLATE "en_US");
INSERT INTO target TABLE source /* ccl_collated */;
REINDEX INDEX ccl_collated;
DROP INDEX ccl_collated;
TRUNCATE target;

-- null prefix column (no cache used for all columns with data)
CREATE INDEX accl ON target USING btree (always_null, country, city, locality);
INSERT INTO target TABLE source /* accl */;
REINDEX INDEX accl;
DROP INDEX accl;
TRUNCATE target;

-- fixed-width columns (all offsets cached), unique but non-unique prefix
CREATE INDEX tnt ON target USING btree (transfer_date, newly_built, transaction);
INSERT INTO target TABLE source /* tnt */;
REINDEX INDEX tnt;
DROP INDEX tnt;
TRUNCATE target;

-- fixed-width columns (all offsets cached), unique but non-unique prefix, caches never unusable due to NULLs
CREATE INDEX ntnt ON target USING btree (always_null, transfer_date, newly_built, transaction);
INSERT INTO target TABLE source /* ntnt */;
REINDEX INDEX ntnt;
DROP INDEX ntnt;
TRUNCATE target;

-- fixed-width columns (all offsets cached), ~high cardinality prefix
CREATE INDEX tn ON target USING btree (transfer_date, newly_built);
INSERT INTO target TABLE source /* tn */;
REINDEX INDEX tn;
DROP INDEX tn;
TRUNCATE target;

-- fixed-width columns (all offsets cached), low cardinality prefix
CREATE INDEX nt ON target USING btree (newly_built, transfer_date);
INSERT INTO target TABLE source /* nt */;
REINDEX INDEX nt;
DROP INDEX nt;
TRUNCATE target;

-- worst-case (no offsets cached, extremely slow compares)
CREATE INDEX worstcase
    ON target
        USING btree (
            coalesce(always_null::text, '') COLLATE "en_US", -- 1
            coalesce(always_null::text, '') COLLATE "en_US", -- 2
            coalesce(always_null::text, '') COLLATE "en_US", -- 3
            coalesce(always_null::text, '') COLLATE "en_US", -- 4
            coalesce(always_null::text, '') COLLATE "en_US", -- 5
            coalesce(always_null::text, '') COLLATE "en_US", -- 6
            coalesce(always_null::text, '') COLLATE "en_US", -- 7
            coalesce(always_null::text, '') COLLATE "en_US", -- 8
            coalesce(always_null::text, '') COLLATE "en_US", -- 9
            coalesce(always_null::text, '') COLLATE "en_US", -- 10
            coalesce(always_null::text, '') COLLATE "en_US", -- 11
            coalesce(always_null::text, '') COLLATE "en_US", -- 12
            coalesce(always_null::text, '') COLLATE "en_US", -- 13
            coalesce(always_null::text, '') COLLATE "en_US", -- 14
            coalesce(always_null::text, '') COLLATE "en_US", -- 15
            coalesce(always_null::text, '') COLLATE "en_US", -- 16
            coalesce(always_null::text, '') COLLATE "en_US", -- 17
            coalesce(always_null::text, '') COLLATE "en_US", -- 18
            coalesce(always_null::text, '') COLLATE "en_US", -- 19
            coalesce(always_null::text, '') COLLATE "en_US", -- 20
            coalesce(always_null::text, '') COLLATE "en_US", -- 21
            coalesce(always_null::text, '') COLLATE "en_US", -- 22
            coalesce(always_null::text, '') COLLATE "en_US", -- 23
            coalesce(always_null::text, '') COLLATE "en_US", -- 24
            coalesce(always_null::text, '') COLLATE "en_US", -- 25
            coalesce(always_null::text, '') COLLATE "en_US", -- 26
            coalesce(always_null::text, '') COLLATE "en_US", -- 27
            coalesce(always_null::text, '') COLLATE "en_US", -- 28
            coalesce(always_null::text, '') COLLATE "en_US", -- 29
            coalesce(always_null::text, '') COLLATE "en_US", -- 30
            coalesce(always_null::text, '') COLLATE "en_US", -- 31
            transaction -- 32, for random ordering of the tuples:
                        -- the TID tie-breaker would otherwise make the index
                        -- append-only, which wouldn't be fun.
        );
INSERT INTO target TABLE source /* worstcase */;
REINDEX INDEX worstcase;
DROP INDEX worstcase;
TRUNCATE target;

COMMIT;
